'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var react = require('react');
var axios = _interopDefault(require('axios'));
var hash = _interopDefault(require('hash-object'));
var debounce = _interopDefault(require('lodash.debounce'));

const { CancelToken } = axios;

/**
 * The object returned by the useAPI hook.
 * @typedef {Object} useAPIOutput
 * @property {Object|undefined} data - The data attribute from the axios response.
 * @property {Object|undefined} response - The axios response.
 * @property {Object|undefined} error - The axios error object if an error occurs.
 * @property {boolean} isLoading - Indicates if their is a pending API call.
 * @property {setDataFunc} setData - Set the response data object.
 */

/**
 * `setData` property of `useAPIOutput`
 * @typedef {function} setDataFunc
 * @param {Object[]} newData - New data array that overwrites current data.
 */

/**
 * React hook used to make a an API call using axios.
 *
 * Allows you to pass an [axios config object](https://github.com/axios/axios#request-config), for complete control of the request being sent.
 *
 * @param {string} url - URL that the API call is made to.
 * @param {Object} config={} - Axios config object passed to the axios.request method.
 * @returns {useAPIOutput} output
 */
function useAPI(url, config = {}) {
  const [state, setState] = react.useState({
    response: undefined,
    error: undefined,
    isLoading: true
  });

  const configHash = hash(config);

  react.useEffect(() => {
    setState({ ...state, isLoading: true });

    const source = CancelToken.source();
    axios(url, {
      ...config,
      cancelToken: source.token
    })
      .then(response => {
        setState({ error: undefined, response, isLoading: false });
      })
      .catch(error => {
        if (axios.isCancel(error)) {
          console.log('Request canceled by cleanup', error.message);
        } else {
          setState({ error, response: undefined, isLoading: false });
        }
      });

    return () => {
      source.cancel('useEffect cleanup.');
    };
  }, [url, configHash]);

  const { response, error, isLoading } = state;

  function setData(newData) {
    // Used to update state from component
    const newResponse = { ...response, data: newData };
    setState({ ...state, response: newResponse });
  }
  const data = response ? response.data : undefined;
  return { data, response, error, isLoading, setData };
}

/**
 * The object returned by the useParams hook.
 * @typedef {Object} useParamsOutput
 * @property {Object} params - The current params to be used when making an API call.
 * @property {boolean} isStale - Is their a debounced params update waiting to timeout. (Are we waiting for the user to stop typing)
 * @property {setParamsFunc} setParams - Function used to set new parameters
 * @property {updateParamsFunc} updateParams - Function used to update current parameters
 * @property {setParamsFunc} debouncedSetParams - Debounced call made to `setParams`
 * @property {updateParamsFunc} debouncedUpdateParams - Debounced call made to `updateParams`
 */

/**
 * `setParams` property of `useParamsOutput`
 * @typedef {function} setParamsFunc
 * @param {Object} newParams - New params object that overwrites the current params.
 */

/**
 * `updateParams` property of `useParamsOutput`
 * @typedef {function} updateParamsFunc
 * @param {Object} paramsUpdate - Partial update to be merged with current params.
 */

/**
 * React hook to keep query parameters in state.
 *
 * Used in conjunction with the other hooks to filter and paginate APIs.
 *
 * Includes the ability the debounce an update, which is useful for delaying API calls while the user is typing.
 *
 * @param {Object} initialParams - The initial parameters to keep in states
 * @param {number} debounceWait=500 - The time to debounce the params update when calling debouncedUpdateParams
 * @returns {useParamsOutput} output
 */
function useParams(initialParams = {}, debounceWait = 500) {
  const [params, setParams] = react.useState(initialParams);
  const [isStale, setIsStale] = react.useState(false);

  function updateParams(updatedParams) {
    const newParams = { ...params, ...updatedParams };
    setParams(newParams);
  }

  const debouncedSetParams = react.useMemo(
    () =>
      debounce(newParams => {
        setIsStale(false);
        setParams(newParams);
      }, debounceWait),
    []
  );

  return {
    params,
    isStale,
    setParams,
    debouncedSetParams: newParams => {
      setIsStale(true);
      debouncedSetParams(newParams);
    },
    updateParams,
    debouncedUpdateParams: updatedParams => {
      setIsStale(true);
      debouncedSetParams({ ...params, ...updatedParams });
    }
  };
}

let pageSize = 40;

/**
 * Default paginator, where the parameter `offset` is used to page through results.
 * @param config {Object} - Axios config object
 * @param paginationState {Object} - Object kept internally to keep track of pagination
 * @return output {Object[]}
 * @return output.updatedConfig {Object} - Config object including pagination changes
 * @return output.updatedPaginationState {Object} - Updated pagination state
 */
function offsetPaginator(config, paginationState) {
  const { params = {} } = config;
  const { offset = 0 } = paginationState;
  const newOffset = offset + pageSize;
  const updatedParams = { ...params, offset: newOffset };

  const updatedConfig = { ...config, params: updatedParams };
  const updatedPaginationState = { offset: newOffset };

  return [updatedConfig, updatedPaginationState];
}

/**
 * Default `responseToItems` function. Assumes that the API response is an array of items.
 * @param response {Object} - Axios response object
 * @return output {Object}
 * @return output.items {Object[]} - Items extracted from the response
 * @return output.hasMore {boolean} - Are there more items available?
 */
function responseToData(response) {
  const items = response.data;
  const hasMore = items.length === pageSize;
  return [items, hasMore];
}

const { CancelToken: CancelToken$1 } = axios;

/**
 * Paginator function used to alter the axios config object, in order to fetch the next page.
 * @typedef {function} paginatorFunc
 * @param config {Object} - Axios config object
 * @param paginationState {Object} - Object kept internally to keep track of pagination
 * @return output {Object[]} - Return tuple \[updatedConfig: Object, updatedPaginationState: Object\]
 */

/**
 * Function used to extract items from the API response.
 * @typedef {function} responseToItemsFunc
 * @param response {Object} - Axios response object
 * @return output {Object} - Return tuple \[items: Object[], hasMore: boolean\]
 */

/**
 * The object returned by the useInfAPI hook.
 * @typedef {Object} useInfAPIOutput
 * @property {Object[]} items - Items provided by the API
 * @property {Object|undefined} error - The axios error object is an error occurs.
 * @property {boolean} isLoading - Indicates if their is a pending API call for the **first** page of items.
 * @property {boolean} isPaging - Indicates if their is a pending API call for the **any** page of items.
 * @property {setItemsFunc} setItems - Set the items being kept in state
 * @property {responseToItemsFunc} fetchPage - Function called from the component in order to fetch the next page
 */

/**
 * `setItems` property of `useInfAPIOutput`
 * @typedef {function} setItemsFunc
 * @param {Object[]} newItems - New items array that overwrites current data.
 */

/**
 * React hook used by an infinite scrolling component to make API calls using axios.
 *
 * Allows you to pass an [axios config object](https://github.com/axios/axios#request-config), for complete control of the request being sent.
 *
 * By default it will paginate using a query param `offset`, and assumes that the API returns an array of items.
 *
 * If this is not appropriate for your API, then you will need to provide your own `paginator` and `responseToItems` functions.
 *
 * @param {string} url - URL that the API call is made to.
 * @param {Object} config={} - Axios config object passed to the axios.request method.
 * @param {paginatorFunc} paginator=offsetPaginator - Function used to update the config object in order to paginate
 * @param {function} responseToItems=responseToData - Function used to extract an array of items from response object.
 * @returns {useInfAPIOutput} output
 */
function useInfAPI(
  url,
  config = {},
  paginator = offsetPaginator,
  responseToItems = responseToData
) {
  const [state, setState] = react.useState({
    items: [],
    paginationState: {},
    error: undefined,
    isLoading: true,
    isPaging: true,
    hasMore: false,
    source: CancelToken$1.source()
  });

  const configHash = hash(config);

  const { items, paginationState } = state;

  function callAPI(isLoading) {
    const activePaginationState = isLoading ? {} : paginationState; // Reset pagination when config object changes.
    const [updatedConfig, updatedPaginationState] = paginator(config, activePaginationState);

    setState({
      ...state,
      isLoading,
      isPaging: true,
      paginationState: updatedPaginationState
    });
    axios(url, {
      ...updatedConfig,
      cancelToken: state.source.token
    })
      .then(response => {
        const [pageItems, hasMore] = responseToItems(response);
        if (typeof pageItems === typeof []) {
          setState({
            ...state,
            items: items.concat(pageItems),
            error: undefined,
            isLoading: false,
            isPaging: false,
            hasMore,
            paginationState: updatedPaginationState
          });
        } else {
          console.log("Warning: responseToItems didn't return an array.");
        }
      })
      .catch(error => {
        if (axios.isCancel(error)) {
          console.log('Request canceled by cleanup', error.message);
        } else {
          setState({ ...state, error, isLoading: false, isPaging: false });
        }
      });
  }

  react.useEffect(() => {
    setState({ ...state, source: CancelToken$1.source() });
    callAPI(true);
    return () => {
      state.source.cancel('useEffect cleanup.');
    };
  }, [url, configHash]);

  const { error, isPaging, isLoading, hasMore } = state;
  return {
    items,
    error,
    isPaging,
    isLoading,
    hasMore,
    setItems: newItems => setState({ ...state, items: newItems }),
    fetchPage: () => callAPI(false)
  };
}

exports.useAPI = useAPI;
exports.useInfAPI = useInfAPI;
exports.useParams = useParams;
